var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { BigNumberString, ConsentFactoryContractError, EVMContractAddress, IpfsCID, MarketplaceListing, TransactionResponseError, UnixTimestamp, } from "@snickerdoodlelabs/objects";
import { ethers } from "ethers";
import { injectable } from "inversify";
import { ResultAsync, errAsync, okAsync } from "neverthrow";
import { BaseContract } from "../implementations/BaseContract.js";
import { ContractsAbis, } from "../interfaces/index.js";
let ConsentFactoryContract = class ConsentFactoryContract extends BaseContract {
    constructor(providerOrSigner, contractAddress) {
        super(providerOrSigner, contractAddress, ContractsAbis.ConsentFactoryAbi.abi);
        this.providerOrSigner = providerOrSigner;
        this.contractAddress = contractAddress;
    }
    getContractAddress() {
        return this.contractAddress;
    }
    //#region Consent
    //#region Contract
    createConsent(ownerAddress, baseUri, overrides) {
        return this.writeToContract("createConsent", [ownerAddress, baseUri], overrides);
    }
    setListingDuration(listingDuration, overrides) {
        return this.writeToContract("setListingDuration", [listingDuration], overrides);
    }
    setMaxTagsPerListing(maxTagsPerListing, overrides) {
        return this.writeToContract("setMaxTagsPerListing", [maxTagsPerListing], overrides);
    }
    registerStakingToken(stakingToken, overrides) {
        return this.writeToContract("registerStakingToken", [stakingToken], overrides);
    }
    adminRemoveListings(tag, stakingToken, removedSlot, overrides) {
        return this.writeToContract("adminRemoveListing", [tag, stakingToken, removedSlot], overrides);
    }
    blockContentObject(stakingToken, contentAddress, overrides) {
        return this.writeToContract("blockContentObject", [stakingToken, contentAddress], overrides);
    }
    unblockContentObject(stakingToken, contentAddress, overrides) {
        return this.writeToContract("unblockContentObject", [stakingToken, contentAddress], overrides);
    }
    //#endregion Contract
    estimateGasToCreateConsent(ownerAddress, baseUri, name) {
        return ResultAsync.fromPromise(this.contract.estimateGas["createConsent"](ownerAddress, baseUri, name), (e) => {
            return this.generateError(e, `Failed to estimate gas with error: ${e}`);
        }).map((estimatedGas) => {
            // TODO: confirm buffer value
            // Increase estimated gas buffer by 10%
            return (estimatedGas * 110n) / 100n;
        });
    }
    getDeployedConsents() {
        const eventFilter = this.contract.filters.ConsentContractDeployed();
        return ResultAsync.fromPromise(this.contract.queryFilter(eventFilter), (e) => {
            return this.generateError(e, "Unable to call contract.queryFilter() for ConsentContractDeployed event");
        }).map((events) => {
            const consents = [];
            events.forEach((event) => {
                if (event instanceof ethers.EventLog) {
                    if (event.args.consentAddress != null) {
                        consents.push(EVMContractAddress(event.args.consentAddress));
                    }
                }
            });
            return consents;
        });
    }
    getListingsByTag(tag, stakingToken, removeExpired) {
        // We get the total number of slots by calling getTagTotal()
        // And if we query the 2^256 - 1 slot by calling getListingDetail(), its previous member variable will point to the highest ranked listing for that tag
        return this.getTagTotal(tag, stakingToken).andThen((tagTotal) => {
            // The max slot
            const highestRankListingSlot = BigNumberString(ethers.MaxUint256.toString());
            // Fetch from the highest to lowest listings
            return this.getListingsForward(tag, stakingToken, highestRankListingSlot, 
            // Since we do not have getListing anymore, we wont immediately be able to get the highest slot (ie. .next of uint256)
            // So to get the whole list, we start with uint256 and work our way down
            // getListingsForward() on the contract returns from first slot queried and works its way down
            // So we need to add one more slot to account for all tags as the first one will be taken by uint256
            // this.getListingsForward function then filters out uint256 and 0 slot values by checking its .next and .previous values.
            tagTotal + 1, removeExpired);
        });
    }
    getAddressOfConsentCreated(txRes) {
        return txRes.wait().andThen((receipt) => {
            // Get the hash of the event
            const event = "ConsentContractDeployed(address,address)";
            const eventHash = ethers.keccak256(ethers.toUtf8Bytes(event));
            // Filter out for the ConsentContractDeployed event from the receipt's logs
            // returns an array
            const consentDeployedLog = receipt.logs.filter((_log) => _log.topics[0] == eventHash);
            // If there are no logs, return an error
            if (consentDeployedLog.length == 0) {
                return errAsync(new TransactionResponseError("No ConsentContractDeployed event found in the transaction receipt"));
            }
            // access the data and topics from the filtered log
            const data = consentDeployedLog[0].data;
            const topics = consentDeployedLog[0].topics;
            // Declare a new interface
            const iface = new ethers.Interface([
                "event ConsentContractDeployed(address indexed owner, address indexed consentAddress)",
            ]);
            // Decode the log from the given data and topic
            const decodedLog = iface.decodeEventLog("ConsentContractDeployed", data, topics);
            const deployedConsentAddress = decodedLog.consentAddress;
            return okAsync(deployedConsentAddress);
        });
    }
    //#endregion Consent
    //#region Content
    getGovernanceToken() {
        return ResultAsync.fromPromise(this.contract.getGovernanceToken(), (e) => {
            return this.generateError(e, "Unable to call getGovernanceToken()");
        });
    }
    isStakingToken(stakingToken) {
        return ResultAsync.fromPromise(this.contract.isStakingToken(stakingToken), (e) => {
            return this.generateError(e, "Unable to call isStakingToken()");
        });
    }
    listingDuration() {
        return ResultAsync.fromPromise(this.contract.listingDuration(), (e) => {
            return this.generateError(e, "Unable to call listingDuration()");
        }).map((duration) => {
            return Number(duration);
        });
    }
    maxTagsPerListing() {
        return ResultAsync.fromPromise(this.contract.maxTagsPerListing(), (e) => {
            return this.generateError(e, "Unable to call maxTagsPerListing()");
        }).map((num) => {
            return Number(num);
        });
    }
    getTagTotal(tag, stakedToken) {
        return ResultAsync.fromPromise(this.contract.getTagTotal(tag, stakedToken), (e) => {
            return this.generateError(e, "Unable to call getTagTotal()");
        }).map((count) => {
            return Number(count);
        });
    }
    getListingsForward(tag, stakingToken, startingSlot, numberOfSlots, removeExpired) {
        return ResultAsync.fromPromise(this.contract.getListingsForward(tag, stakingToken, startingSlot, numberOfSlots, removeExpired), (e) => {
            return this.generateError(e, "Unable to call getListingsForward()");
        }).map(([cids, listings]) => {
            return listings
                .map((listing, index) => {
                return new MarketplaceListing(BigNumberString(listing.previous.toString()), BigNumberString(listing.next.toString()), listing.consentContract, UnixTimestamp(Number(listing.timeExpiring)), IpfsCID(cids[index]), listings[index + 1] != null && listing.next != 0n
                    ? BigNumberString(listings[index + 1].previous.toString())
                    : listings[index - 1] != null
                        ? BigNumberString(listings[index - 1].next.toString())
                        : startingSlot, tag);
            })
                .filter((listing) => {
                // Filter out slots with previous and next zeros
                return listing.previous != "0" || listing.next != "0";
            });
        });
    }
    getListingsBackward(tag, stakingToken, startingSlot, numberOfSlots, removeExpired) {
        return ResultAsync.fromPromise(this.contract.getListingsForward(tag, stakingToken, startingSlot, numberOfSlots, removeExpired), (e) => {
            return this.generateError(e, "Unable to call getListingsForward()");
        }).map(([cids, listings]) => {
            return listings
                .map((listing, index) => {
                return new MarketplaceListing(BigNumberString(listing.previous.toString()), BigNumberString(listing.next.toString()), listing.consentContract, UnixTimestamp(Number(listing.timeExpiring)), IpfsCID(cids[index]), listings[index + 1] != null && listing.previous != ethers.MaxUint256
                    ? BigNumberString(listings[index + 1].next.toString())
                    : listings[index - 1] != null
                        ? BigNumberString(listings[index - 1].previous.toString())
                        : startingSlot, tag);
            })
                .filter((listing) => {
                // Filter out slots with previous and next zeros
                return listing.previous != "0" || listing.next != "0";
            });
        });
    }
    computeFee(slot) {
        return ResultAsync.fromPromise(this.contract.computeFee(slot), (e) => {
            return this.generateError(e, "Unable to call computeFee()");
        });
    }
    // External function meant to be called by anyone to boot an expired listing
    removeExpiredListings(tag, stakingToken, slots, overrides) {
        return this.writeToContract("removeExpiredListings", [tag, stakingToken, slots], overrides);
    }
    //#endregion Content
    // #region Domains- ERC-7529
    addDomain(domain, overrides) {
        return this.writeToContract("addDomain", [domain], overrides);
    }
    removeDomain(domain, overrides) {
        return this.writeToContract("removeDomain", [domain], overrides);
    }
    checkDomain(domain) {
        return ResultAsync.fromPromise(
        // returns array of domains
        this.contract.checkDomain(domain), (e) => {
            return this.generateError(e, "Unable to call checkDomain()");
        });
    }
    // #endregion Domains- ERC-7529
    generateContractSpecificError(msg, e, transaction) {
        return new ConsentFactoryContractError(msg, e, transaction);
    }
};
ConsentFactoryContract = __decorate([
    injectable(),
    __metadata("design:paramtypes", [Object, String])
], ConsentFactoryContract);
export { ConsentFactoryContract };
// I listingStruct { at the place where we're using it, and don't have to export here
// Alternative option is to get the deployed Consent addresses through filtering event ConsentContractDeployed() event
/* // TODO: Replace Promise<any> with correct types returned from ConsentContractDeployed() and queryFilter()
  public getConsentsDeployedByOwner(
    ownerAddress: EVMAccountAddress,
  ): ResultAsync<EVMContractAddress[], ConsentFactoryContractError> {
    return ResultAsync.fromPromise(
      this.contract.filters.ConsentContractDeployed(ownerAddress) as Promise<any>,
      (e) => {
        return new ConsentFactoryContractError(
          "Unable to call filters.ConsentContractDeployed()",
          (e as IBlockchainError).reason,
          e,
        );
      },
    )
      .andThen((_logs) => {
        return ResultAsync.fromPromise(
          this.contract.queryFilter(_logs) as Promise<any>,
          (e) => {
            return new ConsentFactoryContractError(
              "Unable to call filters.ConsentContractDeployed()",
              (e as IBlockchainError).reason,
              e,
            );
          },
        );
      })
      .map((logs) => {
        return logs.map((log) => log.args.consentAddress);
      });
  }
} */
//# sourceMappingURL=ConsentFactoryContract.js.map